/*
 *
 * Copyright (C) 2004 Texas Instruments.
 *
 * ----------------------------------------------------------------------------
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ----------------------------------------------------------------------------
 Modifications:
 ver. 1.0: Oct 2005, Swaminathan S
 *
 */

#include <common.h>
#include "soc.h"
#include "timer.h"
#include <watchdog.h>

/* Use Timer 3&4 (Timer 2) */
#define TIMER_BASE_ADDR	CSL_TMR_1_REGS

davinci_timer_reg    *davinci_timer = (davinci_timer_reg *) TIMER_BASE_ADDR;

#ifdef CONFIG_HW_WATCHDOG
static void initwdt(void);
#else
static inline void initwdt(void) { }
#endif

/* Timer Initialize */
void inittimer(void)
{
        /* disable Timer 1 & 2 timers */
        davinci_timer->tcr = 0;

        /* Set timers to unchained dual 32 bit timers, Unreset timer34 */
        davinci_timer->tgcr = 0x0;
        davinci_timer->tgcr = 0x6;

        /* Program the timer12 counter register - set the prd12 for right count */
        davinci_timer->tim34 = 0;

        /* The timer is programmed to expire after 0xFFFFFFFF ticks */
        davinci_timer->prd34 = 0xFFFFFFFF;

        /* Enable timer34 */
        davinci_timer->tcr = (0x80 << 16); /* Timer34 continously enabled, Timer12 disabled */

	initwdt();
}

/************************************************************
********************** Reset Processor **********************
************************************************************/
#define WDT_BASE_ADDR          CSL_WDT_1_REGS


void reset_processor(void)
{
        davinci_timer_reg *davinci_wdt = (davinci_timer_reg *) WDT_BASE_ADDR;
        davinci_wdt->tgcr   = 0x00000008;
        davinci_wdt->tgcr  |= 0x00000003;
        davinci_wdt->tim12  = 0x00000000;
        davinci_wdt->tim34  = 0x00000000;
        davinci_wdt->prd12  = 0x00000000;
        davinci_wdt->prd34  = 0x00000000;
        davinci_wdt->tcr   |= 0x00000040;
        davinci_wdt->wdtcr |= 0x00004000;
        davinci_wdt->wdtcr  = 0xA5C64000;
        davinci_wdt->wdtcr  = 0xDA7E4000;

	/* Write an invalid key just in case the watchdog was already
	 * configured */
        davinci_wdt->wdtcr  = 0x0000000;
}

#ifdef CONFIG_HW_WATCHDOG

static void initwdt(void)
{
        davinci_timer_reg *davinci_wdt = (davinci_timer_reg *) WDT_BASE_ADDR;
	/* disable timer */
        davinci_wdt->tcr   = 0x0;
	/* reset timer */
        davinci_wdt->tgcr   = 0x0;

	/* configure timer2 as 64-bit */
        davinci_wdt->tgcr   = 0x8;
	/* release timer from reset */
        davinci_wdt->tgcr  |= 0x3;
	/* enable watchdog timer */
        davinci_wdt->wdtcr |= 0x4000;

	/* Set the timer value, clock tick is 27 MHz */
#if (CFG_WATCHDOG_TIMEOUT > 159) || (CFG_WATCHDOG_TIMEOUT <= 0)
#error Maximum watchdog timeout is 159 seconds
#endif
        davinci_wdt->prd12  = CFG_WATCHDOG_TIMEOUT * 27000000U;

	/* enable the timer */
        davinci_wdt->tcr   |= 0x80;
	/* put watchdog in pre-active state */
        davinci_wdt->wdtcr =
		0xA5C64000 | (davinci_wdt->wdtcr & 0xFFFF);
	/* put watchdog in active state */
        davinci_wdt->wdtcr =
		0xDA7E4000 | (davinci_wdt->wdtcr & 0xFFFF);

	/* watchdog is active ! */
}

void hw_watchdog_reset(void)
{
        davinci_timer_reg *davinci_wdt = (davinci_timer_reg *) WDT_BASE_ADDR;

        davinci_wdt->wdtcr =
		0xA5C64000 | (davinci_wdt->wdtcr & 0xFFFF);
        davinci_wdt->wdtcr =
		0xDA7E4000 | (davinci_wdt->wdtcr & 0xFFFF);
}

#endif /* CONFIG_HW_WATCHDOG */
