/*
 * (C) Copyright 2003
 * Texas Instruments.
 * Swaminathan S <swami.iyer@ti.com>
 * (C) Copyright 2007
 * SpinetiX S.A.
 * Diego Santa Cruz <Diego.SantaCruz@spinetix.com>
 * Configuation settings for the SpinetiX Bonsai board.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

/* Chip Configurations */
/*============================================================================*/
#define CFG_DAVINCI
#define CONFIG_ARM926EJS	/* This is an arm926ejs CPU core  	  */
#define CONFIG_SYS_CLK_FREQ	229500000	/* Arm Clock frequency    */
#define CFG_TIMERBASE		0x01C21400	/* use timer 0 		  */
#define CFG_HZ			27000000	/* Timer Input clock freq */	
/*============================================================================*/

/*
 * Flash Boot info
 *============================================================================*/
#define CFG_ENV_IS_IN_FLASH 	1		/* U-Boot env in NOR Flash   */

#ifndef CFG_ENV_IS_IN_FLASH
#define CFG_ENV_IS_IN_NAND 	1               /* U-Boot env in NAND Flash  */
#define CFG_ENV_SECT_SIZE	512		/* Env sector Size */
#define CFG_ENV_SIZE		(16 * 1024)
#else
#define CONFIG_INITRD_TAG  	  1
#define CFG_ENV_SECT_SIZE	CFG_FLASH_SECT_SZ	/* Env sector Size */
#define CFG_ENV_SIZE		CFG_FLASH_SECT_SZ
#define CFG_ENV_ADDR		(CFG_FLASH_BASE + 0x40000)
#define CFG_ENV_ADDR_REDUND	(CFG_ENV_ADDR + CFG_ENV_SECT_SIZE)
#define CFG_ENV_SIZE_REDUND	CFG_FLASH_SECT_SZ
#endif

#define FLASH_ADDR_KERNEL1	0x02080000	/* main kernel image */
#define FLASH_ADDR_KERNEL2	0x02280000	/* secondary kernel image */
#define FLASH_ADDR_INITRD2	0x02480000	/* initrd of 2nd kernel image */
#define FLASH_ADDR_SAFEDATA	0x02FC0000      /* data for failsafe system */

/*
 * NOR Flash Info 
 */
/*============================================================================*/
/* NOR model: Spansion S29GL128N11TFIV2 */
#define CONFIG_CS0_BOOT				/* Boot from Flash 	     */
#define CFG_MAX_FLASH_BANKS	1		/* max number of flash banks */
#define CFG_FLASH_SECT_SZ	0x20000		/* 128KB sect size  */

#ifdef CONFIG_CS0_BOOT
#define PHYS_FLASH_1		0x02000000	/* CS0 Base address 	 */
#endif
#ifdef CONFIG_CS3_BOOT
#define PHYS_FLASH_1		0x00000000	/* Need to update CHECK  */
#endif
#define CFG_FLASH_BASE		PHYS_FLASH_1 	/* Flash Base for U-Boot */
/* #define CONFIG_ENV_OVERWRITE */			/* allow env overwrie 	 */
#define PHYS_FLASH_SIZE		0x1000000	/* Flash size 16MB 	 */
#define CFG_MAX_FLASH_SECT	128		/* max sectors on flash  */

#if(0)
#define CFG_MAX_FLASH_SECT	(PHYS_FLASH_SIZE/CFG_FLASH_SECT_SZ)
#endif
#define CFG_FLASH_ERASE_TOUT	(20*CFG_HZ)	/* Timeout for Flash Erase */
#define CFG_FLASH_WRITE_TOUT	(20*CFG_HZ)	/* Timeout for Flash Write */
/*============================================================================*/

/*
 * Memory Info 
 */
/*============================================================================*/
/* 2x Micron 1 Gbit DDR2 (MT47H64M16HR-5E) */
#define CFG_MALLOC_LEN		(5*1024*1024)   /* malloc () len */
#define CFG_GBL_DATA_SIZE	256		/* reserved for initial data */
#define CFG_ALT_MEMTEST				/* use better memory test */
#define CFG_MEMTEST_START	0x82000000	/* memtest start address, 32 MB */
#define CFG_MEMTEST_END		0x90000000-4	/* 256 MB (i.e. 256-32 RAM test */
#define CFG_MEMTEST_SCRATCH	CFG_MEMTEST_END
#define CONFIG_NR_DRAM_BANKS	1		/* we have 1 bank of DRAM */
#define PHYS_SDRAM_1		0x80000000	/* DDR Start 		  */
#define PHYS_SDRAM_1_SIZE	0x10000000	/* DDR size 256MB 	  */
#define CONFIG_STACKSIZE	(1*1024*1024)	/* big stack		  */
#define BOOTCOUNT_ADDR		0x0000bff8	/* last two 32 bit words of TCM */
/*============================================================================*/

/*
 * Serial Driver info
 */
/*============================================================================*/
#define CFG_NS16550			/* Include NS16550 as serial driver */
#define CFG_NS16550_SERIAL
#define CFG_NS16550_REG_SIZE 	4		/* NS16550 register size */
#define CFG_NS16550_COM1 	0X01C20000	/* Base address of UART0  */
#define CFG_NS16550_CLK 	27000000	/* Input clock to NS16550 */
#define CONFIG_CONS_INDEX	1		/* use UART0 for console  */
#define CONFIG_BAUDRATE		115200		/* Default baud rate      */
#define CFG_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }
/*============================================================================*/

/* U-Boot Configurations */
/*============================================================================*/
/*
 * If we are developing, we might want to start armboot from ram
 * so we MUST NOT initialize critical regs like mem-timing ...
 */
/*#undef CONFIG_INIT_CRITICAL             undef for developing */

/*#define MMU_DEBUG */                              /* define for MMU debug output */
#undef 	CONFIG_USE_IRQ				/* we don't need IRQ/FIQ */
#define CONFIG_MISC_INIT_R
#define CONFIG_HW_WATCHDOG			/* define to enable watchdog */
#define CFG_WATCHDOG_TIMEOUT	90U		/* Timeout in seconds */
#define CONFIG_BOOTDELAY	  20     	/* Boot delay before OS boot*/
#define CONFIG_BOOTFILE		"uImage"	/* file to load */
#define CFG_LONGHELP				/* undef to save memory     */
#define CFG_PROMPT	"Bonsai # "		/* Monitor Command Prompt   */
#define CFG_CBSIZE	1024			/* Console I/O Buffer Size  */
#define CFG_PBSIZE	(CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print buffer sz */
#define CFG_MAXARGS	16		/* max number of command args   */
#define CFG_BARGSIZE	CFG_CBSIZE	/* Boot Argument Buffer Size    */
#undef	CFG_CLKS_IN_HZ			/* Clock info are in HZ */
#define CFG_LOAD_ADDR	0x82000000	/* default load address of various load commands */

#define CONFIG_BOOT_RETRY_TIME	-1	/* enable resume bootdelay countdown if inactive */
#define CONFIG_BOOT_RETRY_MIN	5	/* minimum time to wait for resuming */
#define CONFIG_RESET_TO_RETRY		/* required with Hush shell */
#define CONFIG_ZERO_BOOTDELAY_CHECK	/* check even if boot delay is set to 0 */

/* #define CFG_AUTO_COMPLETE */               /* enable TAB autocompletion, not compatible with Hush shell */
#define CFG_HUSH_PARSER                 /* enable the Hush shell, Bourne like */
#define CFG_PROMPT_HUSH_PS2 "> "        /* the secondary Hush command prompt */
#define CONFIG_VERSION_VARIABLE		/* put version info into 'ver' variable */
#define CONFIG_TIMESTAMP		/* time functions and image timestamp info */
#define CONFIG_BOOTCOUNT_LIMIT		/* alternate boot on repeated boot failure */

#define CONFIG_RTC_PCF8563		/* RTC is RX-8564LC, register compatible with PCF8563 */
#define CFG_I2C_RTC_ADDR	0x51	/* The I2C address of the RTC */

#define CONFIG_SILENT_CONSOLE		/* silent console support */
#define CONFIG_SILENT_CONSOLE_STDIN	/* apply silent mode to stdin also */
#define CFG_DEVICE_NULLDEV		/* include 'nulldev' for I/O */

#define CFG_BONSAI_SILENT_WAIT 1000	/* cumulative msecs to wait
					 * for silent mode override */
#define CFG_BONSAI_RECOVERY_WAIT 5000	/* cumulative msecs to wait
					 * for recovery mode */

/* perform pre-boot checks and extra boot command */
#define CONFIG_PREBOOT							\
	"if test X${needsetup} = Xyes; then "				\
	"echo '***************************************'; "		\
	"echo '*    Pre-Boot checks not passed !!!   *'; "		\
	"echo '*  Attempting to load and run setup.  *'; "		\
	"echo '***************************************'; "		\
	"sleep 1 && "							\
	"setenv autoload no && "					\
	"setenv netretry no && "					\
	"setenv bootdelay 3 && "					\
	"dhcp && "							\
	"if tftpboot 0x82000000 /srv/uboot/setup.scr; then "		\
	"  echo 'Loaded setup script, executing...'; "			\
	"  autoscr; "							\
	"  if test X${setup} != Xok; then "				\
	"    echo 'Setup script failed !!!'; "				\
	"    bootm 0x02000000; "					\
	"  else "							\
	"    echo 'Setup script terminated OK'; "			\
	"  fi; "							\
	"  setenv bootcmd; "						\
	"fi; "								\
	"fi"

#define CONFIG_ETHADDR		00:1d:50:00:00:00
#define CONFIG_OVERWRITE_ETHADDR_ONCE

/* NOTE: the default EMAC address 00:1d:50:00:00:00 used above is
 * intended for manufacturing testing and initialization in a "single
 * Bonsai device per LAN / DHCP server" setting and has been
 * explicitely reserved for this purpose. */

/* NOTE: if the setup script executes we change the bootcmd so that
 * autoboot will not continue. On success it is changed to an empty
 * value (i.e. no effect), on failure it is changed to a bogus bootm
 * command so that failure is indicated by the LEDs. A setup script
 * indicates success by setting the variable 'setup' to 'ok', this
 * variable is not necessarily an environment variable (unfortunately
 * exit codes from autoscr do not seem to propagate to caller). */


/* Version information */
#ifndef BUILD_TAG
#define BUILD_TAG "<none>"
#endif
#define CONFIG_IDENT_STRING " - build: " BUILD_TAG

/*
 *  I2C Configuration 
 */
#define CONFIG_HARD_I2C
#define CFG_I2C_SPEED 100000
#define CFG_I2C_SLAVE 10
#define CONFIG_DRIVER_DAVINCI_I2C
#define CFG_I2C_NO_WRITE_CMD		/* disable all I2C write commands */

/* some I2C devices require await time between repeated a stop
 * condition and the next start condition. Times are as follows:
 * - I2C cycle time at 100 kHz: 10 usec.
 * - RX-8564LC RTC: 1.3 usec
 * - CDCE clock synth: 4.7 usec, 100 usec typical PLL lock time
 * - TPS power converter: 1.3 usec
 * - Core EEPROM: 1.3 usec; 5 ms write cycle time
 * - Video filter: 4.7 usec
 * - VGA encoder: ?
 * - Video decoder: 1.3 usec
 * - HDMI encoder: ?
 * - Carrier EEPROM: 1.2 usec; 5 ms write cycle time
 */
#define CFG_I2C_WAIT_USEC		5 /* half a clock cycle is I2C spec */
#define CFG_EEPROM_PAGE_WRITE_DELAY_MS	5

/* macro to read the 32 bit timer Timer 2 */
#define READ_TIMER (0xFFFFFFFF - (*(volatile ulong *)(CFG_TIMERBASE + 0x14)))

/* Linux Information */

#define LINUX_BOOT_PARAM_ADDR	0x80000100	/* Set the Boot location at the
						 * start of DDR + 0x100
						 */

#define LINUX_MEM		(120*1024*1024)	/* from 120 MB up reserved for DSP and CMEM */

#define CONFIG_CMDLINE_TAG	  1	/* enable passing of ATAGs  */
#define CONFIG_SETUP_MEMORY_TAGS  1
#define CONFIG_INITRD_TAG         1
#define CONFIG_REVISION_TAG       1
#define CONFIG_SERIAL_TAG         1

#define CFG_AUTOLOAD "no"

/* #define BONSAI_TPS_WORKAROUND */	/* define to enable TPS workaround */
#define BONSAI_CDCE_BYPASS	/* set CDCE to 27 MHz bypass on
				 * selected HW revisions */

#define CONFIG_BOOTCOMMAND						\
	"echo '*************************************************'; "	\
	"echo '* U-boot environment and data not installed !!! *'; "	\
	"echo '*       ATTEMPTING TO BOOT FAILSAFE IMAGE       *'; "	\
	"echo '*************************************************'; "	\
	"echo ''; "							\
	"sleep 2; "							\
	"setenv bootargs root=/dev/ram0 rw ip=off panic=2 oops=panic; "	\
	"test X${stdout} = Xserial && "					\
	"  setenv bootargs $bootargs console=ttyS0,$baudrate; "		\
	"bootm $addrsafekrnl $addrsafedisk; "				\
	"echo '*************************************************'; "	\
	"echo '*     FAILSAFE IMAGE MISSING OR CORRUPTED !!!   *'; "	\
	"echo '*        CONTACT SUPPORT FOR ASSISTANCE         *'; "	\
	"echo '*************************************************'; "	\

/*============================================================================*/

/* LED configurations */
/*============================================================================*/

#define CONFIG_STATUS_LED		/* enable LED support */
#define CONFIG_BOARD_SPECIFIC_LED	/* LEDs are board specific */
#define STATUS_LED_PAR			/* keep status_led.h happy */

/* NOTE: The LEDs are the SPEED and LINK LEDs attached to the Ethernet PHY. */
/* They are controlled via MDIO by the PHY */

/* NOTE: blinking LEDs are not supported since that requires interrupt support */

#ifndef __ASSEMBLY__
typedef unsigned short led_id_t;
void __led_init(led_id_t mask, int state);
void __led_toggle(led_id_t mask);
void __led_set(led_id_t mask, int state);
#endif

/* Link LED, green */
#define STATUS_LED_BIT		(1 << 1)
#define STATUS_LED_STATE	STATUS_LED_ON
#define STATUS_LED_PERIOD	(CFG_HZ / 2)

/* Speed LED, red */
#define STATUS_LED_BIT1		(1 << 2)
#define STATUS_LED_STATE1	STATUS_LED_OFF
#define STATUS_LED_PERIOD1	(CFG_HZ / 2)

#define STATUS_LED_GREEN	0
#define STATUS_LED_RED		1

#define CONFIG_SHOW_BOOT_PROGRESS	/* show boot progress using LEDs */

/*============================================================================*/

/*
 * Network & Ethernet Configuration
 */
/*============================================================================*/
#define CONFIG_DRIVER_TI_EMAC

#define CFG_FAULT_MII_ADDR	1	/* The PHY MII address */
#define CFG_FAULT_ECHO_LINK_DOWN	/* Use red LED for link down */
#define CONFIG_BOOTP_MASK	(CONFIG_BOOTP_DEFAULT | CONFIG_BOOTP_DNS | CONFIG_BOOTP_DNS2 | CONFIG_BOOTP_NTPSERVER | CONFIG_BOOTP_TFTPSERVER | CONFIG_BOOTP_SEND_LEASETIME)
#define CONFIG_LEASETIME	1200	/* request DHCP leases for 20 mins. only */
#define CONFIG_TFTPSERVER_HACK		/* use TFTP server name as IP to */
					/* workaround MS DHCP servers */
#define CONFIG_NET_RETRY_COUNT  10
/*============================================================================*/

/*============================================================================*/

/* NAND Flash stuff */
/*============================================================================*/
#ifdef CFG_ENV_IS_IN_NAND
#define CONFIG_COMMANDS		(CONFIG_CMD_DFL | CFG_CMD_PING | CFG_CMD_DHCP | CFG_CMD_NAND)
#define CONFIG_SKIP_LOWLEVEL_INIT       /* needed for booting from NAND as UBL
					 * bootloads u-boot.  The low level init
					 * is configured by the UBL.
					 */
#define CFG_NAND_ADDR           0x02000000

#define CFG_MAX_NAND_DEVICE     1	/* Max number of NAND devices */
#define SECTORSIZE              512

#define ADDR_COLUMN             1
#define ADDR_PAGE               2
#define ADDR_COLUMN_PAGE        3

#define NAND_ChipID_UNKNOWN     0x00
#define NAND_MAX_FLOORS         1
#define NAND_MAX_CHIPS          1
#define CFG_ENV_OFFSET	        0x100000        /* environment starts here  */

#define WRITE_NAND_COMMAND(d, adr) do {*(volatile u8 *)0x02000010 = (u8)d;} while(0)
#define WRITE_NAND_ADDRESS(d, adr) do {*(volatile u8 *)0x0200000A = (u8)d;} while(0)
#define WRITE_NAND(d, adr) do {*(volatile u8 *)0x02000000 = (u8)d;} while(0)
#define READ_NAND(adr) (*(volatile u8 *)0x02000000)
#define NAND_WAIT_READY(nand) while (!((*(volatile u32 *)0x01E00064) & 1))

#define NAND_NO_RB          1

#define NAND_CTL_CLRALE(nandptr) do {} while(0)
#define NAND_CTL_SETALE(nandptr) do {} while(0)
#define NAND_CTL_CLRCLE(nandptr) do {} while(0)
#define NAND_CTL_SETCLE(nandptr) do {} while(0)
#define NAND_DISABLE_CE(nand) do {*(volatile u32 *)0x01E00060 &= ~0x01;} while(0)
#define NAND_ENABLE_CE(nand) do {*(volatile u32 *)0x01E00060 |= 0x01;} while(0)
#else
#define CONFIG_COMMANDS							\
	(CONFIG_CMD_DFL |						\
	 CFG_CMD_PING |							\
	 CFG_CMD_DHCP |							\
	 CFG_CMD_I2C |							\
	 CFG_CMD_AUTOSCRIPT |						\
	 CFG_CMD_DATE |							\
	 CFG_CMD_SNTP |							\
	 CFG_CMD_MII)
#endif

#include <cmd_confdefs.h>

/* KGDB support */
/*============================================================================*/
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CONFIG_KGDB_BAUDRATE	115200	/* speed to run kgdb serial port */
#define CONFIG_KGDB_SER_INDEX	1	/* which serial port to use */
#endif
#endif /* __CONFIG_H */
